/**
 * Multi-layout management with Golden Layout.
 */
const GoldenLayout = require('golden-layout');
const state = require('../app-state');

// Golden Layout initial config.
let config = {
    settings:{
        hasHeaders: true,
        constrainDragToContainer: true,
        reorderEnabled: true,
        selectionEnabled: false,
        popoutWholeStack: false,
        blockedPopoutsThrowError: true,
        closePopoutsOnUnload: true,
        showPopoutIcon: false,
        showMaximiseIcon: true,
        showCloseIcon: false
    },
    content: [{
        type: 'row',
        isClosable: false,
        content: [{
            type:'column',
            isClosable: false,
            content: [{
                type:'component',
                isClosable: false,
                componentName: 'Angles',
                componentState: { text: 'Angles' }
            },
            {
                type:'component', 
                isClosable: false,   
                componentName: 'Scattering Parameters',
                componentState: { text: 'Scattering Parameters' }
            },
            {
                type:'component', 
                isClosable: false,   
                componentName: 'Sample',
                componentState: { text: 'Sample' }
            },
            {
                type:'component', 
                isClosable: false,   
                componentName: 'Braggs Peaks',
                componentState: { text: 'Braggs Peaks' }
            },
            {
                type:'component', 
                isClosable: false,   
                componentName: 'Scattering Plane',
                componentState: { text: 'Scattering Plane' }
            },
            {
                type:'component', 
                isClosable: false,   
                componentName: 'Trajectory Settings',
                componentState: { text: 'Trajectory Settings' }
            },
            {
                type:'component', 
                isClosable: false,   
                componentName: 'Grid Settings',
                componentState: { text: 'Grid Settings' }
            }]
        },
        {
            type:'component',
            isClosable: false,
            componentName: 'Reciprocal View',
            componentState: { text: 'Reciprocal View' }
        },
        {
            type:'column',
            isClosable: false,
            content: [{
                type:'component',    
                isClosable: false,
                componentName: 'Unitcell View',
                componentState: { text: 'Unitcell View' }
            },
            {
                type:'component', 
                isClosable: false,   
                componentName: 'Atoms',
                componentState: { text: 'Atoms' }
            },
            {
                type:'component', 
                isClosable: false,   
                componentName: 'S(q,w)',
                componentState: { text: 'S(q,w)' }
            },
            {
                type:'component', 
                isClosable: false,
                componentName: 'Trajectory',
                componentState: { text: 'Trajectory' }
            }]
        }]
    }]
};

let serializer = null;

// Get the electron serializer or the web serializer.
if (window.process && window.process.versions['electron']) {
    const ElectronMultiLayoutSerializer = require('../electron/electron-multi-layout-serializer');
    serializer = new ElectronMultiLayoutSerializer();
}
else {
    const WebMultiLayoutSerializer = require('../web/web-multi-layout-serializer');
    serializer = new WebMultiLayoutSerializer();
}

let savedConfig = serializer.read();

if (savedConfig != null) {
    // Create the layout with the saved config.
    myLayout = new GoldenLayout(savedConfig);
}
else {
    console.log('No layout config file');

    // Create the layout with the initial config.
    myLayout = new GoldenLayout(config);
}

// Record the state changed event.
myLayout.on('stateChanged', function() {

    if (!state.mode.spyMode) {
        let state = JSON.stringify(myLayout.toConfig());
        serializer.write(state);
    }
    
});


module.exports = myLayout;