const THREE = require('three');

/**
 * 3D shape of arrow. Is implemented with Three.js. Used in reciprocal-view-3d.js
 */
class Arrow3D {

    constructor(color) {

        //(radiusTop, radiusBottom, height, radiusSegments
        this.cylinderHeight = 20;
        this.geometry = new THREE.CylinderGeometry( 0.25, 0.25, this.cylinderHeight, 32 );
        this.material = new THREE.MeshStandardMaterial( {color: color} );
        this.cylinder = new THREE.Mesh( this.geometry, this.material );
        this.cylinder.position.y = 10;

        //radius, height, radialSegments, heightSegments
        this.coneHeight = this.cylinderHeight/6;
        //radius, height, radialSegments, heightSegments
        this.geometry = new THREE.ConeGeometry(1, this.coneHeight, 500);
        this.cone = new THREE.Mesh(this.geometry, this.material);
        this.cone.position.y = this.cylinderHeight;

        this.intermediateGroup = new THREE.Group();
        this.intermediateGroup.add(this.cone)
        this.intermediateGroup.add(this.cylinder)
        this.intermediateGroup.rotateX(Math.PI/2);
        this.intermediateGroup.scale.set(1, 1/(this.cylinderHeight + this.coneHeight/2), 1);

        this._group = new THREE.Group();
        this._group.add(this.intermediateGroup);
    }

    get object3D() {
        return this._group;
    }

    setColor(value) {
        this.material.color.set(value);
    }
}

// Export the constructor.
module.exports = Arrow3D;