const {scalarWorldToView, vector3WorldToView, vector3ViewToWorld} = require('../../utils/conversion');
const THREE = require('three');

class SnapSphere {
    constructor() {
        this._snapGrid = null;
        this._gridVisible = true;

        // Link the sphere to a snap grid
        PubSub.publish('register-sphere', {
            gridName: "mainGrid",
            sphere: this
        });

        let geometry = new THREE.SphereGeometry(1, 8, 8);
        let material = new THREE.MeshBasicMaterial({color: 0x00ff00, opacity: 0.5, transparent: true, depthTest: false});

        // Create the sphere mesh 
        this._mesh = new THREE.Mesh(geometry, material);
        this._mesh.renderOrder = 2;
    }

    set snapGrid(value) { this._snapGrid = value; }

    setColor(value) {
        this._mesh.material.color.set(value);
    }

    get mesh() { return this._mesh; }
    get position() { return this._mesh.position; }

    // Snap the position to the nearest point
    snap(newPosition) {
        // If register-sphere fail.
        if(this._snapGrid === null) {
            console.log("Warning snap on null grid");
            return;
        }
        
        newPosition = this._snapGrid.snap(newPosition);

        this._mesh.position.copy(newPosition);
    }
}

module.exports = SnapSphere;