const MenuEvent = require('../menu-event');

class WebMenuEvent extends MenuEvent {
    constructor() {
        super();

        this._listeners = {};

        this._togglePeaksInPlane = false;
        this._toggle2DView = false;
        this._toggleEnergyView = false;
        this._toggleVRView = false;
        this._fullscreen = false;

        $('#menuSnapMode').click(() => { this.dispatch('change-mode', 'snap'); });
        $('#menuFreeMode').click(() => { this.dispatch('change-mode', 'free'); });
        $('#menuTogglePeaks').click(() => { this._togglePeaksInPlane ^= 1; this.dispatch('toggle-peaks-in-plane', this._togglePeaksInPlane); });
        $('#menuToggle2D').click(() => { this._toggle2DView ^= 1; this.dispatch('toggle-2d-view', this._toggle2DView); });
        $('#menuToggleEnergy').click(() => { this._toggleEnergyView ^= 1; this.dispatch('toggle-energy-view', this._toggleEnergyView); });
        $('#menuToggleVR').click(() => {
            this._toggleVRView ^= true;
            this.dispatch('toggle-vr-view', this._toggleVRView);
        });
        
        $("#menuShowPlane").click(() => { this.dispatch('show-panel', 'scattering-plane'); });
        $("#menuShowScattering").click(() => { this.dispatch('show-panel', 'input'); });
        $("#menuShowAngles").click(() => { this.dispatch('show-panel', 'angles'); });
        $("#menuShowPeaks").click(() => { this.dispatch('show-panel', 'bragg-peaks'); });
        $("#menuShowSample").click(() => { this.dispatch('show-panel', 'sample'); });
        $("#menuShowAtoms").click(() => { this.dispatch('show-panel', 'atoms'); });
        $("#menuShowUnitCell").click(() => { this.dispatch('show-panel', 'unitcell-view-3d'); });
        $("#menuShowSQW").click(() => { this.dispatch('show-panel', 'exp-data'); });
        $("#menuShowConfiguration").click(() => { this.dispatch('show-panel', 'configuration'); });
        $("#menuShowTrajectory").click(() => { this.dispatch('show-panel', 'trajectory'); });
        $("#menuShowGrid").click(() => { this.dispatch('show-panel', 'grid'); });

        document.addEventListener('fullscreenchange', showNav);
        document.addEventListener('webkitfullscreenchange', showNav);
        document.addEventListener('mozfullscreenchange', showNav);
        document.addEventListener('MSFullscreenChange', showNav);

        let self = this;
        function showNav(event) {
            self._fullscreen ^= true;

            if (self._fullscreen) {
                $('#navBar').hide();
            } else {
                $('#navBar').show();
            }
        }
    }

    on(event, listener) {
        if (!this._listeners[event]) {
            this._listeners[event] = [listener];
        } else {
            this._listeners[event].push(listener);
        }
    }

    dispatch(event, data) {
        if (this._listeners[event]) {
            for (let listener of this._listeners[event]) {
                listener(event, data);
            }
        }
    }
}

module.exports = WebMenuEvent;