const Model = require('./model');
const THREE = require('three');
const vEXPController = require('../vexp-controller');

class PlaneModel extends Model {
    constructor(ax, ay, az, bx, by, bz) {
        super();

        this._ax = ax;
        this._ay = ay;
        this._az = az;
        this._bx = bx;
        this._by = by;
        this._bz = bz;

        this._aVector = new THREE.Vector3(this._ax, this._ay, this._az);
        this._bVector = new THREE.Vector3(this._bx, this._by, this._bz);
        this.calcABVector();
    }

    // Plane parameter setters
    set ax(value) { 
        this._ax = value;
        this._aVector.x = value;
        
        vEXPController.setPlanes(this._ax, this._ay, this._az, this._bx, this._by, this._bz);
        this.calcABVector();
    }

    set ay(value) { 
        this._ay = value;
        this._aVector.y = value;
        
        vEXPController.setPlanes(this._ax, this._ay, this._az, this._bx, this._by, this._bz);
        this.calcABVector();
    }

    set az(value) { 
        this._az = value;
        this._aVector.z = value;
        
        vEXPController.setPlanes(this._ax, this._ay, this._az, this._bx, this._by, this._bz);
        this.calcABVector();
    }

    set bx(value) { 
        this._bx = value;
        this._bVector.x = value;
        
        vEXPController.setPlanes(this._ax, this._ay, this._az, this._bx, this._by, this._bz);
        this.calcABVector();
    }

    set by(value) { 
        this._by = value;
        this._bVector.y = value;
        
        vEXPController.setPlanes(this._ax, this._ay, this._az, this._bx, this._by, this._bz);
        this.calcABVector();
    }

    set bz(value) {
        this._bz = value;
        this._bVector.z = value;
        
        vEXPController.setPlanes(this._ax, this._ay, this._az, this._bx, this._by, this._bz);
        this.calcABVector();
    }

    // Plane parameter getters
    get ax() { return this._ax; }
    get ay() { return this._ay; }
    get az() { return this._az; }
    get bx() { return this._bx; }
    get by() { return this._by; }
    get bz() { return this._bz; }
    get aVector() { return this._aVector; }
    get bVector() { return this._bVector; }
    get abVector() { return this._abVector; }

    // Set all plane parameters.
    setPlane(ax, ay, az, bx, by, bz) {
        this._ax = ax;
        this._ay = ay;
        this._az = az;
        this._bx = bx;
        this._by = by;
        this._bz = bz;
        this._aVector.set(ax, ay, az);
        this._bVector.set(bx, by, bz);

        vEXPController.setPlanes(this._ax, this._ay, this._az, this._bx, this._by, this._bz);
        this.calcABVector();
    }

    /**
     * Calculate AB vector (plan normal).
     */
    calcABVector() {
        this._abVector = new THREE.Vector3(0, 0, 1).applyMatrix3(vEXPController.uMatrix_1);
    }

    toObject() {
        return {
            ax: this._ax,
            ay: this._ay,
            az: this._az,
            bx: this._bx,
            by: this._by,
            bz: this._bz,
            aVector: { x: this._aVector.x, y: this._aVector.y, z: this._aVector.z },
            bVector: { x: this._bVector.x, y: this._bVector.y, z: this._bVector.z },
            abVector: { x: this._abVector.x, y: this._abVector.y, z: this._abVector.z }
        };
    }

    /**
     * Save the model data into a config object (the saver object).
     */
    save(saver) {
        saver.plane = this.toObject();
    }

    /**
     * Restore the model state saved in the saver object (the configuration object).
     */
    restore(saver) {
        let plane = saver.plane;

        this.setPlane(
            plane.ax, plane.ay, plane.az,
            plane.bx, plane.by, plane.bz
        );
    }
}

module.exports = PlaneModel;